# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

STORAGE_ACCOUNT_NAME = "fakename"
STORAGE_ACCOUNT_KEY = "fakekey"
PREMIUM_STORAGE_FILE_ACCOUNT_NAME = "fakename"
PREMIUM_STORAGE_FILE_ACCOUNT_KEY = "fakekey"
SECONDARY_STORAGE_ACCOUNT_NAME = "fakename"
SECONDARY_STORAGE_ACCOUNT_KEY = "fakekey"

TENANT_ID = "00000000-0000-0000-0000-000000000000"
CLIENT_ID = "00000000-0000-0000-0000-000000000000"
CLIENT_SECRET = "00000000-0000-0000-0000-000000000000"

ACCOUNT_URL_SUFFIX = 'core.windows.net'
RUN_IN_LIVE = "False"
SKIP_LIVE_RECORDING = "True"

PROTOCOL = "https"
